#!/usr/bin/env python

# to release, this script follows these steps:
# change version in const.py
# make sure gtkexcepthook is imported in epidermis.py and creator.py
# make sure const.DEBUG is False
# make sure debian/changelog is correct
# make sure CHANGELOG.txt is correct
# run bzr builddeb
# sign the .deb and the .tar.gz
# create release using this script, use changelog from CHANGELOG.txt
#	prompt for summary
# add .tar.gz then .deb to downloads for release
# rm ../epidermis_*
# bzr builddeb -S
# dput epidermis epidermis_*changes

from launchpadlib.launchpad import Launchpad, STAGING_SERVICE_ROOT, OAUTH_REALM
import os

new_version = "0.4"
cachedir = os.path.expanduser("~/.cache/launchpadlib")
ppa = "epidermis" # for dput
realm = OATH_REALM

from launchpadlib.credentials import Credentials
import gtk
import datetime, time
import sys
import subprocess

# edit const.py VERSION
root = os.path.dirname(os.path.dirname(os.path.abspath(os.path.realpath(__file__))))
ff = open(os.path.join(root,"epidermis/const.py"),"r")
output = ""
change = True
for line in ff.readlines():
    if line.strip().replace(" ","")[:8] == "VERSION=":
        if line.split("=")[1].strip() == "\"%s\"" % new_version:
            change = False
            break
        output += "VERSION = \"%s\"\n" % new_version
    elif line[:6] == "DEBUG=":
        if line.split("=")[1].strip()[:5] != "False":
            print >> sys.stderr, "const.py DEBUG does not equal False, exiting"
            sys.exit(1)
        else:
            output += line
    else:
        output += line
ff.close()
if change:
    ff = open(os.path.join(root,"epidermis/const.py"),"w")
    ff.write(output)
    ff.close()

# make sure gtkexcepthook is imported in epidermis.py and creator.py
for filename in ("epidermis.py","creator.py"):
    ff = open(os.path.join(root,"epidermis/%s"% filename),"r")
    found = False
    for line in ff.readlines():
        if line.strip()[:20] == "import gtkexcepthook":
            found = True
            break
    if not found:
        print >> sys.stderr, "import gtkexcepthook not found in %s, exiting" % filename
        sys.exit(1)
    ff.close()
    
# make sure debian/changelog is correct
ff = open(os.path.join(root,"debian/changelog"),"r")
line = ff.readlines()[0]
deb_version = line.split("(")[1].split(")")[0]
if deb_version.split("-")[0] != new_version:
    print >> sys.stderr, "debian/changelog version is not correct, exiting"
    sys.exit(1)

# check CHANGELOG.txt
# also, cache it, it will be added to the release
ff = open(os.path.join(root,"CHANGELOG.txt"),"r")
line = ff.readlines()[0]
if line.strip() != new_version + ":":
    print >> sys.stderr, "CHANGELOG.txt has wrong version number, exiting"
    sys.exit(1)
changelog = ""
ff.seek(0)
for line in ff.readlines()[1:]:
    changelog += line
    if line.strip() == "":
        break
ff.close()
    
# run bzr builddeb
rr = subprocess.call(["bzr","builddeb"],cwd=root)
if rr != 0:
    print >> sys.stderr, "builddeb failed"
    sys.exit(rr)

# sign the .deb and the .tar.gz
rr = subprocess.call("rm *.asc",shell=True,cwd=os.path.dirname(root))
rr = subprocess.call(["gpg","--armor","--sign","--detach-sig","epidermis_%s_all.deb" % deb_version],cwd=os.path.dirname(root))
rr += subprocess.call(["gpg","--armor","--sign","--detach-sig","epidermis_%s.tar.gz" % deb_version],cwd=os.path.dirname(root))
if rr != 0:
    print >> sys.stderr, "gpg failed"
    sys.exit(rr)

# log in to Launchpad
if os.path.exists(os.path.expanduser("~/launchpadcreds.txt")):
    credentials = Credentials()
    credentials.load(open(os.path.expanduser("~/launchpadcreds.txt")))
    launchpad = Launchpad(credentials, realm, cachedir)
else:
    launchpad = Launchpad.get_token_and_login('Epidermis script', realm, cachedir)
    launchpad.credentials.save(file(os.path.expanduser("~/launchpadcreds.txt"), "w"))

# find correct Launchpad milestone
ep = launchpad.projects['epidermis']
milestones = ep.all_milestones
release_milestone = None
for milestone in milestones:
    if milestone.name == new_version:
        release_milestone = milestone
        break
    print milestone.name
if release_milestone is None:
    print >> sys.stderr, "cannot find launchpad milestone: %s, exiting" % new_version
    sys.exit(1)

# ask for release summary
# make GUI
win = gtk.Window()
win.set_title("Summary")
win.set_default_size(300,400)
vbox = gtk.VBox(2)
vbox.set_homogeneous(False)
scr = gtk.ScrolledWindow()
textbox = gtk.TextView()
textbox.set_wrap_mode(gtk.WRAP_WORD)
button = gtk.Button()
button.set_label("OK")
def hm(aa=None):
    win.hide()
    gtk.main_quit()
button.connect("clicked",hm)
win.connect("destroy",lambda aa:sys.exit())
win.add(vbox)
scr.add(textbox)
vbox.pack_start(scr,True,True)
vbox.pack_start(button,False,False)
win.show_all()
gtk.main()
print "window closed"
# get result
buff = textbox.get_buffer()
description = buff.get_text(buff.get_start_iter(),buff.get_end_iter())

# find correct Launchpad release, or create it
today = datetime.datetime.today()
try:
	release = release_milestone.createProductRelease(date_released=today, changelog=changelog,\
        release_notes=description)
except:
	release = release_milestone.release

# upload source tarball and .deb file
print "If you receive an UTF8 error, you need to update launchpadlib to 1.5.1"
release.add_file(content_type="application/x-gzip", description="Source code",\
    file_content=open(os.path.join(os.path.dirname(root),"epidermis_%s.tar.gz" % deb_version),"rb").read(), \
    file_type="Code Release Tarball", filename="epidermis_%s.tar.gz" % deb_version, \
    signature_content=open(os.path.join(os.path.dirname(root),"epidermis_%s.tar.gz.asc" % deb_version),"r").read(),\
    signature_filename="epidermis_%s.tar.gz.asc" % deb_version)
release.add_file(content_type="application/x-debian-package", description="Debian package for Ubuntu",\
    file_content=open(os.path.join(os.path.dirname(root),"epidermis_%s_all.deb" % deb_version),"r").read(),
    file_type="Installer file", filename="epidermis_%s_all.deb" % deb_version,
    signature_content=open(os.path.join(os.path.dirname(root),"epidermis_%s_all.deb.asc" % deb_version),"r").read(),\
    signature_filename="epidermis_%s_all.deb.asc" % deb_version)
    
# save changes
print "saving changes"
release.lp_save()

# prepare packages for PPA
rr = subprocess.call("rm -f epidermis_*",shell=True,cwd=os.path.dirname(root))
rr += subprocess.call(["bzr","builddeb","-S"],cwd=root)
if rr != 0:
    print >> sys.stderr, "bzr builddeb -S failed. exiting"
    sys.exit(rr)
print "epidermis_%s_source.changes" % deb_version
# upload to PPA
rr = subprocess.call(["dput", ppa ,"epidermis_%s_source.changes" % deb_version],cwd=os.path.dirname(root))
print rr

print "Done!"
print "Now announce it on your website!"
