#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# some parts of this file were copied from GursorMaker 0.6.0, which was released under the GPL v2, Copyright (c) 2006 Vincent Paeder
# GursorMaker can be found at: http://gursormaker.sourceforge.net/
# released under the GPLv3


from pigment import PigmentType, Pigment
from pigment_more import DirBasedPigment, DirBasedPigmentType
import pigment
import os, sys
import subprocess
import gconf
import gtk
from glob import glob
from Numeric import fromstring
import Numeric
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import handy
import hashlib
from gettext import gettext as _


class CursorsType(DirBasedPigmentType):
    humanName = {"en":"Cursors"}
    humanPluralName = {"en":"Cursors"}
    codeName = "cursors"
    abbreviation = "cr"
    directoryName = "cursors"
    canGeneratePreview = True

    attachmentTypeHelp = _("The theme file should be the directory that bears the name of theme, and containing a directory named cursors, it should not contain any other directories")
    
    def get_Pigment(self):
        return Cursors

    publicBaseDir = "/usr/share/icons"
    personalBaseDir = "~/.icons"
    
    isDirectoryShared = True
    specificChildren = ("cursors", "cursor.theme")


class Cursors(DirBasedPigment):
    
    
    def __init__(self):
        DirBasedPigment.__init__(self)
        self.type = CursorsType()

    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/desktop/gnome/peripherals/mouse/cursor_theme", self.attDirname)
        progressCallback(1.0)
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if DirBasedPigment.is_installed(self, installationDirectory):
            if os.path.isdir(os.path.join(self.type.publicBasedDir, self.attDirname, "cursors")):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if not DirBasedPigment.is_attachment_correct(self, attachmentIndex):
            return False
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        dir = att.split("/")[-1]
        if dir in ("cursors"):
            return False
        if not os.path.isdir(att):
            return False
        if not os.path.isdir(os.path.join(att,"cursors")):
            return False
        if self.get_state != pigment.STATE_INSTALLED:
            for dir in os.listdir(att):
                if os.path.isdir(dir):
                    if dir != "cursors":
                        return False
        if len(os.listdir(os.path.join(att, "cursors"))) == 0:
            return False
        return True
    
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        # FIXME: cursors is_installed
        try:
            f1 = glob(self.attPath + "/*") 
            f2 = glob("/usr/share/themes/%s/*" % self.attDirname)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        # FIXME: meat_activated
        try:
            f1 = glob(self.attPath + "/*")
            f2 = glob("/usr/share/icons/%s/*" % client.get_string("/desktop/gnome/peripherals/mouse/cursor_theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def generate_preview(self):
        if len(self.attPath) < 1:
            return None
        subprocess.call(["mkdir", "-p", "/tmp/epidermis"])
        arrowFile = None
        for ii in ("default", "left_ptr","arrow",  "arrow.xzf"):
            if os.path.exists(os.path.join(self.attPath,  "cursors",  ii)):
                arrowFile = os.path.join(self.attPath,  "cursors",  ii)
                break
        if arrowFile is None:
            print >> sys.stderr,  "cannot find arrow cursor"
            return None
        subprocess.call(["cp",  arrowFile,  "/tmp/epidermis/arrow"])
        res = self._ReadXCursor("/tmp/epidermis/arrow")
        if res == False:
            return None
        pb = res[3][0]
        if pb is None:
            return None
        else:
            self.preview = pb
            return pb
        
    def _ReadXCursor(self,  filename):
        """Reads an xcursor file and returns list:
        [xhot, yhot, delay, picList]
        on failure, returns False
        Taken from GursorMaker which was released under the GPL v2"""
        try:
            file = open(filename,'r+b')
        except IOError:
            print "IOError"
            return False
        
        magic = file.read(4)
        if magic != 'Xcur':
            return False
        
        header = fromstring(file.read(4))
        version = file.read(4)
        ntoc = fromstring(file.read(4))
        
        ftype = []
        size = []
        position = []
        
        for i in range(ntoc):
            ftype.append(file.read(4))
            size.append(fromstring(file.read(4)))
            position.append(fromstring(file.read(4)))
        
        width = []
        height = []
        delay = []
        pixels = []
        
        for i in range(ntoc):
            file.seek(int(position[i]))
            header = fromstring(file.read(4))
            ctype = file.read(4)
            csubtype = fromstring(file.read(4))
            if ctype != ftype[i] or csubtype != size[i]:
                file.close()
                print >> sys.stderr, "ctype error"
                return False
            if fromstring(ctype) == -131071:
                pass # it's a comment
            elif fromstring(ctype) == -196606:
                version = fromstring(file.read(4))
                width.append(int(fromstring(file.read(4))))
                height.append(int(fromstring(file.read(4))))
                xhot = int(fromstring(file.read(4)))
                yhot = int(fromstring(file.read(4)))
                delay.append(int(fromstring(file.read(4))))
                # images are represented in ARGB, byteswapped format. That is, each word contains in order B, G, R, A
                # pixbufs need RGBA representation
                p = Numeric.swapaxes(Numeric.reshape(fromstring(file.read(4*height[-1]*width[-1]),'b'),(width[-1]*height[-1],4)),0,1)
                pixels.append(Numeric.swapaxes(Numeric.array([p[2],p[1],p[0],p[3]],'b'),0,1).tostring())
                
        file.close()
        
        pics = list()
        for i in range(ntoc):
            try:
                pics.append(gtk.gdk.pixbuf_new_from_data(pixels[i], gtk.gdk.COLORSPACE_RGB, True, 8, width[i], height[i], width[i]*4))
            except:
                pics.append(None)
        
        return [xhot, yhot, delay, pics]
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Icons()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()
